from pylab import *
import spimage
from optparse import OptionParser

if __name__ == "__main__":
    parser = OptionParser(usage="%name -n NUMBER_OF_ITERATIONS -f DIFFRACTION_PATTERN -r REAL_SPACE_MODEL -s SUPPORT [-o OUTPUT_DIR -a AFFIX]")
    parser.add_option("-n", action="store", type="int", default=100, dest="number_of_iterations", help="Number of iterations of ER")
    parser.add_option("-f", action="store", type="string", default=None, dest="pattern", help="Diffraction pattern")
    parser.add_option("-r", action="store", type="string", default=None, dest="real_space", help="Starting real-space model")
    parser.add_option("-s", action="store", type="string", default=None, dest="support", help="Support")
    parser.add_option("-o", action="store", type="string", default=".", dest="output_dir", help="Output directory")
    parser.add_option("-a", action="store", type="string", default="refined", dest="output_affix", help="This name will be added to all output files.")
    options, args = parser.parse_args()

    seed()
    spimage.sp_srand(randint(1e6))
    intensities = spimage.sp_image_read(options.pattern, 0)
    if intensities.shifted == 0:
        amplitudes = spimage.sp_image_shift(intensities)
    else:
        amplitudes = spimage.sp_image_duplicate(intensities, spimage.SP_COPY_ALL)
    spimage.sp_image_dephase(amplitudes)
    spimage.sp_image_to_amplitudes(amplitudes)

    real_space = spimage.sp_image_read(options.real_space, 0)
    support = spimage.sp_image_read(options.support, 0)

    phase_alg = spimage.sp_phasing_er_alloc(spimage.SpNoConstraints)

    sup_alg = spimage.sp_support_array_init(spimage.sp_support_static_alloc(), 20)

    # create phaser
    phaser = spimage.sp_phaser_alloc()
    spimage.sp_phaser_init(phaser, phase_alg, sup_alg, spimage.SpEngineCUDA)
    spimage.sp_phaser_set_amplitudes(phaser, amplitudes)
    spimage.sp_phaser_init_model(phaser, real_space, 0)
    spimage.sp_phaser_init_support(phaser, support, 0, 0)

    #real_space_s = spimage.sp_image_shift(real_space)
    fourier_space = spimage.sp_image_ifftw3(real_space)
    ereal_start = (abs(real_space.image[-bool8(support.image)])**2).sum() / (abs(real_space.image[bool8(support.image)])**2).sum()
    efourier_start = ((abs(fourier_space.image[bool8(amplitudes.mask)]) - abs(amplitudes.image[bool8(amplitudes.mask)]))**2).sum() / (abs(amplitudes.image[bool8(amplitudes.mask)])**2).sum()

    spimage.sp_phaser_iterate(phaser, options.number_of_iterations)

    #model_out = spimage.sp_phaser_model(phaser)
    model_out = spimage.sp_phaser_model(phaser)
    support_out = spimage.sp_phaser_support(phaser)
    fmodel_out = spimage.sp_phaser_fmodel(phaser)
    # ereal_end = (abs(model_out.image[-bool8(support.image)])**2).sum() / (abs(model_out.image[bool8(support.image)])**2).sum()
    # efourier_end = ((abs(fmodel_out.image[bool8(amplitudes.mask)]) - abs(amplitudes.image[bool8(amplitudes.mask)]))**2).sum() / (abs(amplitudes.image[bool8(amplitudes.mask)
    #                                                                                                                                               ])**2).sum()
    ereal_end = spimage.sp_phaser_ereal(phaser)
    efourier_end = spimage.sp_phaser_efourier(phaser)


    spimage.sp_image_write(model_out, "%s/real_space-%s.h5" % (options.output_dir, options.output_affix), 0)
    spimage.sp_image_write(support_out, "%s/support-%s.h5" % (options.output_dir, options.output_affix), 0)
    spimage.sp_image_write(fmodel_out, "%s/fourier_space-%s.h5" % (options.output_dir, options.output_affix), 0)

    print "Ereal:    %g -> %g" % (ereal_start, ereal_end)
    print "Efourier: %g -> %g" % (efourier_start, efourier_end)

